﻿ EnableExplicit
#CANNOT_GET_FREQUENCY						= 1
#CANNOT_GET_START_TIME						= 2
#CANNOT_GET_STOP_TIME						= 3
#NO_PROCESSOR_AVAILABLE_TO_GET_START_TIME	= 4
#NO_PROCESSOR_AVAILABLE_TO_GET_STOP_TIME	= 5
#STOP_TIME_TAKEN_ON_DIFFERENT_PROCESSOR		= 6

#Elapsed_Ticks = 0
#Elapsed_ns = 1
#Elapsed_ms = 2
#Elapsed_sec = 3
#LOOPS = 100

Structure Etime_t

	qStart.q
	qStop.q
	qFreq.q;
    qTicks.q
    	
	lOverHead.l

	lProcessorMask.l;

	bAlreadyStart.b;

	lErrorFlag.l;

EndStructure

Global gEnTimer.Etime_t, dResult.d, glHRT_Error, glMHz, gqFreqHz.q
Global gCPUFreq.q, gqFreq.q

ProcedureDLL GetEnTError()

    ProcedureReturn gEnTimer\lErrorFlag

EndProcedure

ProcedureDLL SetEnTError(SetErr)

    gEnTimer\lErrorFlag = SetErr
    
    ProcedureReturn gEnTimer\lErrorFlag

EndProcedure

Procedure.b EtimeFrequency(*mtime.Etime_t)

	;LARGE_INTEGER Freq;
	Protected qFreq.q;

	If (QueryPerformanceFrequency_(@qFreq))	; ; //Get the frequency of the timer
		*mtime\qFreq = qFreq
		gqFreq = qFreq
		ProcedureReturn #True;
		
	Else
	
		PokeQ(@*mtime\lErrorFlag, #CANNOT_GET_FREQUENCY)
		
		ProcedureReturn #False;
	EndIf
EndProcedure


Procedure GetProcSpeed()
Protected dwMHz = #MAX_PATH, hKey, lError, lMHz

    ;hKey = Space(#MAX_PATH)
    
;// open the key where the proc speed is hidden:
    lError = RegOpenKeyEx_(#HKEY_LOCAL_MACHINE, "HARDWARE\DESCRIPTION\System\CentralProcessor\0", 0, #KEY_READ, @hKey);
    
    If (lError <> #ERROR_SUCCESS)
      ; {// If the key is Not found, tell the user why:
;            FormatMessage(#FORMAT_MESSAGE_FROM_SYSTEM,
;                          NULL,
;                          lError,
;                          0,
;                          Buffer,
;                          _MAX_PATH,
;                          0);
;                AfxMessageBox(Buffer);
            glHRT_Error = lError 
            ProcedureReturn 0; "N/A";
    Else
        ; // query the key:
        lError = RegQueryValueEx_(hKey, "~MHz", #Null, #Null, @lMhz, @dwMHz);

    EndIf
    
    ProcedureReturn lMHz
EndProcedure


ProcedureDLL.l EnTimer(*mtime.Etime_t)
Protected Status.l = #True, Counter.q; 
;LARGE_INTEGER	Counter; 

Protected CurrentProcessHandle, ProcessAffinity, SystemAffinity, AllowProcessAffinity, AffinityMask;
	
	If @*mtime = 0
	    *mtime = gEnTimer
	    
	EndIf
	
	; //
	CurrentProcessHandle = GetCurrentProcess_();
	GetProcessAffinityMask_(CurrentProcessHandle, @ProcessAffinity, @SystemAffinity);

	; //Bit vector representing Processors that the thread can run on
	AllowProcessAffinity = ProcessAffinity & SystemAffinity;


	If (*mtime\bAlreadyStart)	; //Get STOP time
		

		; //Make sure at least one processor is available For this thread
		If (AllowProcessAffinity)  
		
			; //Make sure running on the same processor that gets the START time
			If (*mtime\lProcessorMask & AllowProcessAffinity)
				If (SetProcessAffinityMask_(CurrentProcessHandle, *mtime\lProcessorMask))
				
					If (QueryPerformanceCounter_(@Counter))
						*mtime\qStop = Counter;
						*mtime\qFreq = gqFreq
						
					Else
					
						*mtime\lErrorFlag = #CANNOT_GET_STOP_TIME;
						Status = #False;
						
					EndIf
				Else 
				
					*mtime\lErrorFlag = #NO_PROCESSOR_AVAILABLE_TO_GET_STOP_TIME;
					Status = #False;
					
				EndIf
			Else
			
				*mtime\lErrorFlag = #STOP_TIME_TAKEN_ON_DIFFERENT_PROCESSOR;
				Status = #False;
				
			EndIf
						
		
		Else 
		
			*mtime\lErrorFlag = #NO_PROCESSOR_AVAILABLE_TO_GET_STOP_TIME;
			Status = #False;
			
		EndIf
		
	Else            ; // Get Start Ticks
	
		
		; //Make sure at least one processor is available For this thread
		If (AllowProcessAffinity)  
		
			AffinityMask = 1;

			; //Select the lowest processor ID To run       ; loops are skipped by default
			While (AffinityMask <= AllowProcessAffinity)    ; to include them use directive ;$EnDBG  on same line   
			                                                 
				; //Check If this processor is available
				If (AffinityMask & AllowProcessAffinity)
				
					If (SetProcessAffinityMask_(CurrentProcessHandle, AffinityMask))
					
						*mtime\lProcessorMask = AffinityMask;	; //Save processor mask 
                        ;MessageRequester("1", Str(*mtime\lErrorFlag))

						If (QueryPerformanceCounter_(@Counter))
						
							*mtime\qStart = Counter;
							*mtime\bAlreadyStart = #True;     ; //Already got start time
						    ;MessageRequester("2", Str(*mtime\lErrorFlag))
						    Status = #True
						    *mtime\lOverhead = gEnTimer\lOverHead
						    
						Else
						
							*mtime\lErrorFlag = #CANNOT_GET_START_TIME;
							Status = #False;
							;MessageRequester("3", Str(*mtime\lErrorFlag))
							
						EndIf


						Break;
					EndIf

				EndIf

				AffinityMask = AffinityMask << 1;  ; //Check the next processor
                ;MessageRequester("4", Str(*mtime\lErrorFlag))
                
			Wend                                            ; stop skipping loop after this line

		
		Else 
		
			*mtime\lErrorFlag = #NO_PROCESSOR_AVAILABLE_TO_GET_START_TIME;
			Status = #False;
			;MessageRequester("5", Str(*mtime\lErrorFlag))
		EndIf

	EndIf

	; //Reset the processor affinity
	SetProcessAffinityMask_(CurrentProcessHandle, ProcessAffinity);

    *mtime\qTicks = (*mtime\qStop - *mtime\qStart - *mtime\lOverhead)
    ProcedureReturn @*mtime\qTicks
	
	
EndProcedure

Procedure InitEnTimer(*mtime.Etime_t)
Protected i, mTime.Etime_t
	*mtime\qStart     = 0;
	*mtime\qStop      = 0;
	*mtime\qFreq      = 0;
	*mtime\qTicks     = 0;

	*mtime\lProcessorMask = 1;   ; //Lowest processor ID

	*mtime\bAlreadyStart = #False;

	*mtime\lErrorFlag = 0;

    ;$HRTStart                                              ; tracer directive (start High Resolution Timer)
	If (EtimeFrequency(*mtime))
	    EnTimer(*mTime)
	    
	    For i = 1 To #LOOPS
    	    EnTimer(@mTime)
    	    EnTimer(@mTime)
	    Next i
	    EnTimer(*mTime)
	    gEnTimer\lOverHead = (mtime\qStop - mtime\qStart) / (#LOOPS+2)

	;$HRTStop                                               ; tracer directive (Stop High Resolution Timer and show elapsed)
	    
		ProcedureReturn  #True;
		
	Else 
		ProcedureReturn #False;
		
    EndIf
    
EndProcedure


ProcedureDLL AttachProcess(Instance)

    glMHz = GetProcSpeed()
    gqFreqHz = glMHz * 1000000
    QueryPerformanceFrequency_(@gqFreq)
    
    ;InitPerfTimer()
    InitEnTimer(@gEnTimer)
    
;     MessageRequester("", "")

    ProcedureReturn 0

EndProcedure

ProcedureDLL DetachProcess(Instance)

    ProcedureReturn 0

EndProcedure

ProcedureDLL AttachThread(Instance)

    ProcedureReturn 0

EndProcedure

ProcedureDLL DetachThread(Instance)

    ProcedureReturn 0

EndProcedure

; #Elapsed_Ticks = 0
; #Elapsed_ns = 1
; #Elapsed_ms = 2
; #Elapsed_sec = 3

ProcedureDLL.l GetElapsed(*mtime.Etime_t = 0, lUnit = 0)

    If *mtime = 0
        *mtime = @gEnTimer
        
    EndIf

     dResult = (*mtime\qStop - *mtime\qStart - *mtime\lOverHead)
            
    If lUnit = #Elapsed_ns    ; ns
        dResult = dResult * 1000000000 / (*mtime\qFreq)
        
    ElseIf lUnit = #Elapsed_ms              ; ms
        dResult = dResult * 1000 / (*mtime\qFreq)
        
    ElseIf lUnit = #Elapsed_sec             ; sec  
        dResult = dResult / (*mtime\qFreq)

    EndIf
    
    ProcedureReturn @dResult
    
EndProcedure


CompilerIf 0                                                ; set to 1 to test the code

Define pEnTime.Etime_t

    InitEnTimer(@pEnTime)
    EnTimer(@pEnTime)           ; get start time
    
    Sleep_(1000)                ; wait 1 sec
    
    EnTimer(@pEnTime)           ; get stop time
    
    Debug pEnTime\lOverHead     ; overhead for API calls
    
    GetElapsed(@pEnTime)
    
    Debug pEnTime\qTicks ; / EnTimer\qFreq / 1000000000 
    
    Debug StrD(PeekD(GetElapsed(pEnTime, #Elapsed_Ticks))) +" ticks"; /EnTimer\qFreq / 1000000000 
    Debug StrD(PeekD(GetElapsed(pEnTime, #Elapsed_ns)))+" ns" ; /EnTimer\qFreq / 1000000000 
    Debug StrD(PeekD(GetElapsed(pEnTime, #Elapsed_ms)))+" ms" ; /EnTimer\qFreq / 1000000000 
    Debug StrD(PeekD(GetElapsed(pEnTime, #Elapsed_sec)))+" sec" ; /EnTimer\qFreq / 1000000000 


CompilerEndIf

; IDE Options = PureBasic 4.31 (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 246
; FirstLine = 28
; Folding = --
; EnableThread
; Executable = _$_EnTimer.dll
; CurrentDirectory = .